import javax.microedition.lcdui.*;
import java.util.*;
import javax.microedition.media.*;

public class MetronomeCanvas extends Canvas implements CommandListener { 
    Metronome metronome;
    Timer     timer;
    long      prevTime;
    int       screenWidth;
    int       screenHeight;
    TimerTask task;
    Font      font;
    Display   display;

    static final int WHITE = 0xFFFFFF;
    static final int BLACK = 0;
    static final int FREQ_STEP = 10;
    static final int INPUT_DELAY = 2000;
    static final int MAX_DELAY = 60*1000;

    MetronomeCanvas(Metronome metronome) { 
        this.metronome = metronome;
        timer = new Timer();
	screenWidth = getWidth();
	screenHeight = getHeight();
        font = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_LARGE);
        setCommandListener(this);
        addCommand(Metronome.QUIT_CMD);
        addCommand(Metronome.SETTINGS_CMD);
        addCommand(Metronome.HELP_CMD); 
        display = Display.getDisplay(metronome);
        display.setCurrent(this);
        restart();
    }

    class MetronomeTask extends TimerTask { 
        public void run() { 
            //System.out.println("Metronome fired");
            if ((metronome.flags & Metronome.MELODY) != 0) { 
                try { 
                    Player player = Manager.createPlayer(this.getClass().getResourceAsStream("alarm.wav"), "audio/x-wav");
                    player.start();
                } catch (Exception x) {
                    x.printStackTrace();
                }
            } else { 
                try { 
                    Manager.playTone(metronome.note, metronome.duration, metronome.volume);
                } catch (MediaException x) {
                    //System.out.println("MEdia exception " + x);
                }
            }
            if ((metronome.flags & Metronome.VIBRA) != 0) { 
                display.vibrate(metronome.duration);
            }                    

            //System.out.println("metronome.sound.played");
            repaint();
        }
    }

    void restart() {
        //System.out.println("Restart rate=" + metronome.rate);
        if (task != null) { 
            task.cancel();
            task = null;
        }
        metronome.saveSettings();
        if (metronome.rate != 0) { 
            //System.out.println("Schedule timer with interval=" + (60*1000/metronome.rate));
            task = new MetronomeTask();
            timer.scheduleAtFixedRate(task, 0, 60*1000/metronome.rate);
        } else { 
            repaint();
        }
    }


    protected void paint(Graphics g) {
        //System.out.println("point on=" + on + ", seal=" + seal);
        g.setColor(WHITE); 
        g.fillRect(0, 0, screenWidth, screenHeight);        
        display.flashBacklight(metronome.duration);
        g.setFont(font);        
        g.setColor(BLACK); 
        String rate = Integer.toString(metronome.rate);
        g.drawString(rate, screenWidth/2, screenHeight/2, Graphics.HCENTER|Graphics.BASELINE);
        //System.out.println("End of paint");
    }
            
    public void keyPressed(int keyCode)
    {
        switch (keyCode) 
        {
        case KEY_NUM0:
        case KEY_NUM1:
        case KEY_NUM2:
        case KEY_NUM3:
        case KEY_NUM4:
        case KEY_NUM5:
        case KEY_NUM6:
        case KEY_NUM7:
        case KEY_NUM8:
        case KEY_NUM9:
        {
            long currTime = System.currentTimeMillis();
            if (currTime - prevTime > INPUT_DELAY) { 
                metronome.rate = keyCode - KEY_NUM0;
            } else { 
                metronome.rate = metronome.rate*10 + keyCode - KEY_NUM0;
            }
            prevTime = currTime;            
            restart();
            break;            
        }
        default:
            switch (getGameAction(keyCode)) { 
            case LEFT:
                if (metronome.rate > 0) { 
                    metronome.rate -= 1;
                }
                break;
            case RIGHT:
                metronome.rate += 1;
                break;
            case UP:
                metronome.rate += FREQ_STEP;
                break;
            case DOWN:
                if (metronome.rate >= FREQ_STEP) { 
                    metronome.rate -= FREQ_STEP;
                }
                break;
            default:
                return;
            }
            restart();
        }
    }


    public void commandAction(Command c, Displayable d) {
        if (c == Metronome.QUIT_CMD) { 
            metronome.destroyApp(true);
            metronome.notifyDestroyed();
        } else if (c == Metronome.SETTINGS_CMD) { 
            new SettingsForm(metronome);
        } else if (c == Metronome.HELP_CMD) { 
            new HelpForm(metronome);
        }
    }         
}